// Kalkulator.java - Jednostavan kalkulator u grafickom okruzenju.

import java.awt.*;
import java.awt.event.*;

public class Kalkulator extends Frame {
  
  private long x, y;                         // Registri kalkulatora.
  private Label[] reg = new Label [2];       // Prikaz registara.
  private final int X = 0, Y = 1;            // Indeksi registara.
  private boolean noviBroj = true;           // Unosi se novi broj.
  private int osnova = 10;                   // Osnova brojevnog sistema.
  private Button[] cifre = new Button [16];  // Dugmad s ciframa.
  private boolean zavrsi = false;            // Da li unistiti prozor pri
                                             //   zatvaranju?
  public Kalkulator (int x, int y) {         // SASTAVLJANJE PROZORA:
    super ("Kalkulator");
    setBounds (x, y, 305, 242);
    setResizable (false);
    addWindowListener (new WindowAdapter () {
      public void windowClosing (WindowEvent d)
        { if (zavrsi) dispose (); else setVisible (false); }
    });
    
    { Panel ploca = new Panel();             // - dugmad s ciframa,
      ploca.setLayout (new GridLayout (4, 4));
      add (ploca,  BorderLayout.CENTER);
      CifraAkcija osmatrac = new CifraAkcija ();
      for (int i=0; i<16; i++) {
        ploca.add (cifre[i] = new Button (Integer.toString (i,16)));
        cifre[i].addActionListener (osmatrac);
        cifre[i].setEnabled (i < osnova);
      }
    }
    { String[] ozn = {"2", "8", "10", "16"}; // - radio dugmad s osnovama
      Panel ploca = new Panel ();            //   brojevnih sistema,
      add (ploca,  BorderLayout.WEST);
      ploca.setLayout (new GridLayout (ozn.length, 1));
      ploca.setBackground (Color.yellow);
      CheckboxGroup grupa = new CheckboxGroup ();
      OsnovaPromena osmatrac = new OsnovaPromena ();
      for (int i=0; i<ozn.length; i++) {
         Checkbox radio = new Checkbox (ozn[i]);
         radio.setCheckboxGroup (grupa);
         ploca.add (radio);
         radio.addItemListener (osmatrac);
         radio.setState (Integer.parseInt (ozn[i]) == osnova);
      }
    }
    { Panel ploca = new Panel ();            // - polja teksta za registre,
      ploca.setLayout (new GridLayout (2, 1));
      add (ploca,  BorderLayout.NORTH);
      Color[] podloge = {Color.orange, Color.gray};
      Color[] slova   = {Color.blue, Color.white};
      for (int i=0; i<2; i++) {
        reg[i] = new Label ("0", Label.RIGHT);
        reg[i].setFont (new Font (null, Font.BOLD, 20));
        reg[i].setForeground (slova[i]);
        reg[i].setBackground (podloge[i]);
        ploca.add (reg[i]);
      }
    }
    { String[] ozn = {"=", "0", "+=", "|=", "-=", "^=", // - dugmad sa
         "*=", "&=", "/=", "<<=", "%=", ">>="};         //   operatorima.
      Panel ploca = new Panel();
      add (ploca,  BorderLayout.EAST);
      ploca.setLayout (new GridLayout ((ozn.length+1)/2, 2));
      OperatorAkcija osmatrac = new OperatorAkcija ();
      for (int i=0; i<ozn.length; i++) {
        Button dugme = new Button (ozn[i]);
        dugme.setForeground (Color.blue);
        dugme.setBackground (Color.yellow);
        dugme.addActionListener (osmatrac);
        ploca.add (dugme);
      }
    }
  } // Konstruktor
  
  public Kalkulator () { this (250, 50); }   // Podrazumevani konstruktor.

  private class CifraAkcija implements ActionListener {    // Obrada 
    public void actionPerformed (ActionEvent d) {          //   pritisaka
      String cif = ((Button)d.getSource ()).getLabel ();   //   na cifre.
      String staro = reg[Y].getText();
      reg[Y].setText ((noviBroj || staro.equals("0") ? "" : staro) + cif);
      noviBroj = false;
      try { 
        y = Long.parseLong (reg[Y].getText(), osnova); 
      } catch (NumberFormatException g) {
        reg[Y].setText ("G R E S K A");
        noviBroj = true;
      }
    }
  }
  
  private void prikazi () {                                // Prikaz 
    reg[X].setText (Long.toString (x, osnova));            //   sadrzaja 
    reg[Y].setText (Long.toString (y, osnova));            //   registara.
  }
  
  private class OsnovaPromena implements ItemListener {    // Obrada
    public void itemStateChanged (ItemEvent d) {           //   promene
      String ozn = ((Checkbox)(d.getSource())).getLabel(); //   osnove
      osnova = Integer.parseInt (ozn);                     //   brojevnog
      for (int i=0; i<16; i++)                             //   sistema.
        cifre[i].setEnabled (i < osnova);
      prikazi ();
    }
  }
  
  private class OperatorAkcija implements ActionListener { // Obrada
    public void actionPerformed (ActionEvent d) {          //   pritisaka na
      try {                                                //   operatore.
        switch (((Button)d.getSource()).getLabel().charAt(0)) {
          case '=': x   = y; break;    case '0': y   = 0; break;
          case '+': x  += y; break;    case '|': x  |= y; break;
          case '-': x  -= y; break;    case '&': x  &= y; break;
          case '*': x  *= y; break;    case '^': x  ^= y; break;
          case '/': x  /= y; break;    case '<': x <<= y; break;
          case '%': x  %= y; break;    case '>': x >>= y; break;
        }
        prikazi ();
      } catch (ArithmeticException g) {
        reg[Y].setText ("G R E S K A");
      }
      noviBroj = true;
    }
  }
  
  public long rezultat () { return x; }       // Dohvatanje rezultata.
  
    public static void main (String[] varg) { // Glavni program.
    Kalkulator kalk = new Kalkulator ();
    kalk.setVisible (true);
    kalk.zavrsi = true; 
  }
}